@@ -27,6 +27,10 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
+#ifdef HAVE_CLUTTER
+#include <clutter/clutter.h>
+#endif
+
 #include "ar-debug.h"
 #include "ar-runtime.h"
 #include "ar-string-utils.h"
@@ -242,7 +246,16 @@ clear_slots (AisleriotGame *game,
   for (i = 0; i < n_slots; ++i) {
     ArSlot *slot = game->slots->pdata[i];
 
+#ifdef HAVE_CLUTTER
+    if (slot->slot_renderer) {
+      clutter_actor_destroy (slot->slot_renderer);
+      g_object_unref (slot->slot_renderer);
+    }
+
+    g_byte_array_free (slot->old_cards, TRUE);
+#else
     g_ptr_array_free (slot->card_images, TRUE);
+#endif /* HAVE_CLUTTER */
     g_byte_array_free (slot->cards, TRUE);
 
     g_slice_free (ArSlot, slot);
@@ -405,11 +418,11 @@ game_scm_pre_unwind_handler (void *user_data,
     return SCM_UNDEFINED;
 
   message = cscmi_exception_get_backtrace (tag, throw_args);
-  g_set_error (error,
-               AISLERIOT_GAME_ERROR,
-               GAME_ERROR_EXCEPTION,
-               message ? message
-                       : "A scheme exception occurred, but there was no exception info");
+  g_set_error_literal (error,
+                       AISLERIOT_GAME_ERROR,
+                       GAME_ERROR_EXCEPTION,
+                       message ? message
+                               : "A scheme exception occurred, but there was no exception info");
   return SCM_UNDEFINED;
 }
 
@@ -652,7 +665,11 @@ cscmi_add_slot (SCM slot_data)
   slot->expanded_down = expanded_down != FALSE;
   slot->expanded_right = expanded_right != FALSE;
 
+#ifdef HAVE_CLUTTER
+  slot->old_cards = g_byte_array_sized_new (SLOT_CARDS_N_PREALLOC);
+#else
   slot->card_images = g_ptr_array_sized_new (SLOT_CARDS_N_PREALLOC);
+#endif
 
   slot->needs_update = TRUE;
 
@@ -1776,7 +1793,7 @@ game_scm_new_game (void *user_data)
  */
 static void
 aisleriot_game_new_game_internal (AisleriotGame *game,
-                                  GRand *rand,
+                                  GRand *g_rand,
                                   gboolean count_loss)
 {
   GObject *object = G_OBJECT (game);
@@ -1803,11 +1820,11 @@ aisleriot_game_new_game_internal (AisleriotGame *game,
     update_statistics (game);
   }
 
-  if (rand != NULL) {
+  if (g_rand != NULL) {
     if (game->rand)
       g_rand_free (game->rand);
 
-    game->rand = rand; /* adopted */
+    game->rand = g_rand; /* adopted */
   } else if (game->rand == NULL) {
     game->rand = g_rand_new ();
   }
@@ -1866,9 +1883,9 @@ aisleriot_game_new_game (AisleriotGame *game)
  */
 void
 aisleriot_game_new_game_with_rand (AisleriotGame *game,
-                                   GRand *rand)
+                                   GRand *g_rand)
 {
-  aisleriot_game_new_game_internal (game, rand, TRUE);
+  aisleriot_game_new_game_internal (game, g_rand, TRUE);
 }
 
 /**
@@ -1880,12 +1897,12 @@ aisleriot_game_new_game_with_rand (AisleriotGame *game,
 void
 aisleriot_game_restart_game (AisleriotGame *game)
 {
-  GRand *rand;
+  GRand *g_rand;
 
-  rand = game->saved_rand;
+  g_rand = game->saved_rand;
   game->saved_rand = NULL;
 
-  aisleriot_game_new_game_internal (game, rand, FALSE);
+  aisleriot_game_new_game_internal (game, g_rand, FALSE);
 }
 
 /**
@@ -2170,17 +2187,12 @@ aisleriot_game_get_hint (AisleriotGame *game)
                  aisleriot_game_get_game_file (game));
       break;
 
-    case 4:
-      string1 = SCM_CADR (hint);
-      if (!scm_is_string (string1))
-        break;
-
-      str1 = scm_to_locale_string (string1);
-      scm_dynwind_free (str1);
-      if (!str1)
-        break;
-
-      message = g_strdup_printf (_("You are searching for a %s."), str1);
+    case 4: /* This is deprecated (due to i18n issues) do not use. */
+      g_warning ("This game uses a deprecated hint method (case 4).\n"
+                 "Please file a bug at http://bugzilla.gnome.org "
+                 "including this message and the name of the game "
+                 "you were playing, which is %s.\n",
+                 aisleriot_game_get_game_file (game));
       break;
 
     default:
@@ -2495,3 +2507,41 @@ aisleriot_game_deal_cards (AisleriotGame *game)
   aisleriot_game_end_move (game);
   aisleriot_game_test_end_of_game (game);
 }
+
+#ifdef HAVE_CLUTTER
+
+void
+aisleriot_game_get_card_offset (ArSlot *slot,
+                                guint card_num,
+                                gboolean old_cards,
+                                gint *xoff, gint *yoff)
+{
+  gint n_cards, exposed;
+
+  if (old_cards) {
+    n_cards = (gint) slot->old_cards->len;
+    exposed = (gint) slot->old_exposed;
+  } else {
+    n_cards = (gint) slot->cards->len;
+    exposed = (gint) slot->exposed;
+  }
+
+  if (card_num >= n_cards - exposed) {
+    gint idx = card_num + exposed - n_cards;
+    *xoff = slot->pixeldx * idx;
+    *yoff = slot->pixeldy * idx;
+  } else {
+    *xoff = 0;
+    *yoff = 0;
+  }
+}
+
+void
+aisleriot_game_reset_old_cards (ArSlot *slot)
+{
+  g_byte_array_set_size (slot->old_cards, 0);
+  g_byte_array_append (slot->old_cards, slot->cards->data, slot->cards->len);
+  slot->old_exposed = slot->exposed;
+}
+
+#endif /* HAVE_CLUTTER */
